@@ -48,7 +48,7 @@ typedef struct {
   guint signal_id;
   GTask *task;
   char *request_path;
-  guint cancelled_id;
+  gulong cancelled_id;
 } EmailCall;
 
 static void
@@ -64,8 +64,7 @@ email_call_free (EmailCall *call)
   if (call->signal_id)
     g_dbus_connection_signal_unsubscribe (call->portal->bus, call->signal_id);
 
-  if (call->cancelled_id)
-    g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+  g_clear_signal_handler (&call->cancelled_id, g_task_get_cancellable (call->task));
 
   g_free (call->request_path);
 
@@ -96,11 +95,7 @@ response_received (GDBusConnection *bus,
   guint32 response;
   g_autoptr(GVariant) ret = NULL;
 
-  if (call->cancelled_id)
-    {
-      g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
-      call->cancelled_id = 0;
-    }
+  g_clear_signal_handler (&call->cancelled_id, g_task_get_cancellable (call->task));
 
   g_variant_get (parameters, "(u@a{sv})", &response, &ret);
 
@@ -162,12 +157,7 @@ call_returned (GObject *object,
   ret = g_dbus_connection_call_with_unix_fd_list_finish (bus, NULL, result, &error);
   if (error)
     {
-      if (call->cancelled_id)
-        {
-          g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
-          call->cancelled_id = 0;
-        }
-
+      g_clear_signal_handler (&call->cancelled_id, g_task_get_cancellable (call->task));
       g_task_return_error (call->task, error);
       email_call_free (call);
     }
@@ -256,7 +246,7 @@ compose_email (EmailCall *call)
       for (i = 0; call->attachments[i]; i++)
         {
           g_autoptr(GError) error = NULL;
-          int fd;
+          g_autofd int fd = -1;
           int fd_in;
 
           fd = g_open (call->attachments[i], O_PATH | O_CLOEXEC);
@@ -305,7 +295,7 @@ compose_email (EmailCall *call)
  * @flags: options for this call
  * @cancellable: (nullable): optional [class@Gio.Cancellable]
  * @callback: (scope async): a callback to call when the request is done
- * @data: (closure): data to pass to @callback
+ * @data: data to pass to @callback
  *
  * Presents a window that lets the user compose an email,
  * with some pre-filled information.
