@@ -1,3 +1,27 @@
+/*
+ * Copyright (C) 2018, Matthias Clasen
+ * Copyright (C) 2024 GNOME Foundation, Inc.
+ *
+ * This file is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation, version 3.0 of the
+ * License.
+ *
+ * This file is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: LGPL-3.0-only
+ *
+ * Authors:
+ *    Matthias Clasen <mclasen@redhat.com>
+ *    Hubert Figuière <hub@figuiere.net>
+ */
+
 #include "config.h"
 
 #include <gtk/gtk.h>
@@ -11,6 +35,7 @@
 #include <fcntl.h>
 #include <string.h>
 
+#include "libportal/settings.h"
 #include "libportal/portal-gtk3.h"
 
 #include "portal-test-app.h"
@@ -59,10 +84,16 @@ struct _PortalTestWin
   GtkWidget *open_local_dir;
   GtkWidget *file_chooser_button;
 
+  XdpSettings *settings;
+  GtkWidget *settings_value;
+  GtkWidget *settings_count;
 
   GtkWidget *screencast_label;
   GtkWidget *screencast_toggle;
 
+  GtkWidget *inputcapture_label;
+  GtkWidget *inputcapture_toggle;
+
   GFileMonitor *update_monitor;
   GtkWidget *update_dialog;
   GtkWidget *update_dialog2;
@@ -156,7 +187,7 @@ update_available (XdpPortal *portal,
                   PortalTestWin *win)
 {
   g_message ("Update  available");
-   
+
   gtk_label_set_label (GTK_LABEL (win->update_label), "Update available");
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (win->update_progressbar), 0.0);
 
@@ -188,7 +219,7 @@ update_progress (XdpPortal *portal,
     }
 
   if (status != XDP_UPDATE_STATUS_RUNNING)
-    g_signal_handlers_disconnect_by_func (win->portal, update_progress, win); 
+    g_signal_handlers_disconnect_by_func (win->portal, update_progress, win);
 }
 
 static void
@@ -228,6 +259,45 @@ update_dialog2_response (GtkDialog     *dialog,
     }
 }
 
+static void
+reload_settings (PortalTestWin *win, GtkButton* button)
+{
+  g_autoptr(GError) error = NULL;
+  const char *ns[] = { "org.freedesktop.appearance", NULL };
+  guint32 value = xdp_settings_read_uint (win->settings, ns[0], "contrast", NULL, &error);
+  g_autoptr(GVariant) value2 = NULL;
+
+  if (!error)
+    {
+      guint32 uvalue2;
+      g_autofree char *label = g_strdup_printf ("got %u", value);
+      gtk_label_set_text (GTK_LABEL (win->settings_value), label);
+
+      xdp_settings_read (win->settings, ns[0], "contrast", NULL, &error, "u", &uvalue2);
+      if (!error)
+	g_assert (uvalue2 == value);
+    }
+
+
+  value2 = xdp_settings_read_all_values (win->settings, ns, NULL, NULL);
+  if (value2)
+    {
+      gsize count = g_variant_n_children (value2);
+      g_autofree char *count_label = g_strdup_printf ("%lu", count);
+
+      gtk_label_set_text (GTK_LABEL (win->settings_count), count_label);
+    }
+}
+
+static void
+settings_changed (XdpSettings* settings, const gchar *namespace, const gchar *key, GVariant *value, gpointer data)
+{
+  PortalTestWin *win = data;
+  g_autofree char *label = g_strdup_printf ("chg %s %s", namespace, key);
+
+  gtk_label_set_text (GTK_LABEL (win->settings_value), label);
+}
+
 static void
 portal_test_win_init (PortalTestWin *win)
 {
@@ -284,6 +354,9 @@ portal_test_win_init (PortalTestWin *win)
   xdp_portal_update_monitor_start (win->portal, 0, NULL, NULL, NULL);
   g_signal_connect (win->portal, "update-available", G_CALLBACK (update_available), win);
   g_signal_connect (win->update_dialog2, "response", G_CALLBACK (update_dialog2_response), win);
+
+  win->settings = xdp_portal_get_settings (win->portal);
+  g_signal_connect (win->settings, "changed", G_CALLBACK (settings_changed), win);
 }
 
 static void
@@ -298,7 +371,7 @@ opened_uri (GObject *object,
   gboolean res;
 
   open_dir = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->open_local_dir));
- 
+
   if (open_dir)
     res = xdp_portal_open_directory_finish (portal, result, &error);
   else
@@ -561,6 +634,87 @@ take_screenshot (GtkButton *button,
   xdp_parent_free (parent);
 }
 
+static void
+inputcapture_session_created (GObject *source,
+                              GAsyncResult *result,
+                              gpointer data)
+{
+  XdpPortal *portal = XDP_PORTAL (source);
+  PortalTestWin *win = data;
+  g_autoptr(GError) error = NULL;
+  GList *zones;
+  g_autoptr (GString) s = NULL;
+  XdpInputCaptureSession *ic;
+
+  ic = xdp_portal_create_input_capture_session_finish (portal, result, &error);
+  if (ic == NULL)
+    {
+      g_warning ("Failed to create inputcapture session: %s", error->message);
+      return;
+    }
+  win->session = XDP_SESSION (ic);
+
+  zones = xdp_input_capture_session_get_zones (XDP_INPUT_CAPTURE_SESSION (win->session));
+  s = g_string_new ("");
+  for (GList *elem = g_list_first (zones); elem; elem = g_list_next (elem))
+    {
+      XdpInputCaptureZone *zone = elem->data;
+      guint w, h;
+      gint x, y;
+
+      g_object_get (zone,
+		    "width", &w,
+		    "height", &h,
+		    "x", &x,
+		    "y", &y,
+		    NULL);
+
+      g_string_append_printf (s, "%ux%u@%d,%d ", w, h, x, y);
+    }
+  gtk_label_set_label (GTK_LABEL (win->inputcapture_label), s->str);
+}
+
+static void
+start_input_capture (PortalTestWin *win)
+{
+  g_clear_object (&win->session);
+
+  xdp_portal_create_input_capture_session (win->portal,
+                                           NULL,
+                                           XDP_INPUT_CAPABILITY_POINTER | XDP_INPUT_CAPABILITY_KEYBOARD,
+                                           NULL,
+                                           inputcapture_session_created,
+                                           win);
+}
+
+static void
+stop_input_capture (PortalTestWin *win)
+{
+  if (win->session != NULL)
+    {
+      xdp_session_close (win->session);
+      g_clear_object (&win->session);
+      gtk_label_set_label (GTK_LABEL (win->inputcapture_label), "");
+    }
+}
+
+static void
+capture_input (GtkButton *button,
+               PortalTestWin *win)
+{
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+    start_input_capture (win);
+  else
+    stop_input_capture (win);
+}
+
+static void
+capture_input_release (GtkButton *button,
+                       PortalTestWin *win)
+{
+  /* FIXME */
+}
+
 static void
 session_started (GObject *source,
                  GAsyncResult *result,
@@ -599,7 +753,7 @@ session_started (GObject *source,
 
   gtk_label_set_label (GTK_LABEL (win->screencast_label), s->str);
 }
-  
+
 static void
 session_created (GObject *source,
                  GAsyncResult *result,
@@ -616,7 +770,7 @@ session_created (GObject *source,
       g_warning ("Failed to create screencast session: %s", error->message);
       return;
     }
-  
+
   parent = xdp_parent_new_gtk (GTK_WINDOW (win));
   xdp_session_start (win->session, parent, NULL, session_started, win);
   xdp_parent_free (parent);
@@ -650,7 +804,7 @@ stop_screencast (PortalTestWin *win)
 }
 
 static void
-screencast_toggled (GtkToggleButton *button, 
+screencast_toggled (GtkToggleButton *button,
                     PortalTestWin *win)
 {
   if (gtk_toggle_button_get_active (button))
@@ -726,7 +880,7 @@ compose_email_called (GObject *source,
   PortalTestWin *win = data;
   g_autoptr(GError) error = NULL;
 
-  if (!xdp_portal_compose_email_finish (win->portal, result, &error)) 
+  if (!xdp_portal_compose_email_finish (win->portal, result, &error))
     {
       g_warning ("Email error: %s", error->message);
       return;
@@ -1247,6 +1401,8 @@ portal_test_win_class_init (PortalTestWinClass *class)
   gtk_widget_class_bind_template_callback (widget_class, open_directory);
   gtk_widget_class_bind_template_callback (widget_class, open_local);
   gtk_widget_class_bind_template_callback (widget_class, take_screenshot);
+  gtk_widget_class_bind_template_callback (widget_class, capture_input);
+  gtk_widget_class_bind_template_callback (widget_class, capture_input_release);
   gtk_widget_class_bind_template_callback (widget_class, screencast_toggled);
   gtk_widget_class_bind_template_callback (widget_class, notify_me);
   gtk_widget_class_bind_template_callback (widget_class, print_cb);
@@ -1254,6 +1410,7 @@ portal_test_win_class_init (PortalTestWinClass *class)
   gtk_widget_class_bind_template_callback (widget_class, play_clicked);
   gtk_widget_class_bind_template_callback (widget_class, get_user_information);
   gtk_widget_class_bind_template_callback (widget_class, compose_email);
+  gtk_widget_class_bind_template_callback (widget_class, reload_settings);
   gtk_widget_class_bind_template_callback (widget_class, request_background);
   gtk_widget_class_bind_template_callback (widget_class, set_wallpaper);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, sandbox_status);
@@ -1269,10 +1426,14 @@ portal_test_win_class_init (PortalTestWinClass *class)
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, inhibit_logout);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, inhibit_suspend);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, inhibit_switch);
+  gtk_widget_class_bind_template_child (widget_class, PortalTestWin, inputcapture_label);
+  gtk_widget_class_bind_template_child (widget_class, PortalTestWin, inputcapture_toggle);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, username);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, realname);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, avatar);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, save_how);
+  gtk_widget_class_bind_template_child (widget_class, PortalTestWin, settings_value);
+  gtk_widget_class_bind_template_child (widget_class, PortalTestWin, settings_count);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, screencast_label);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, screencast_toggle);
   gtk_widget_class_bind_template_child (widget_class, PortalTestWin, update_dialog);
