@@ -18,8 +18,7 @@
  * SPDX-License-Identifier: LGPL-3.0-only
  */
 
-#ifndef TEST_H
-#define TEST_H
+#pragma once
 
 #include <QObject>
 
@@ -30,5 +29,3 @@ private Q_SLOTS:
     void testFileChooserPortal();
     void testNotificationPortal();
 };
-
-#endif // TEST_H
